@@ -1,11 +1,11 @@
 name = "labelbox"
-__version__ = "3.4.0"
+__version__ = "3.11.0"
 
 from labelbox.schema.project import Project
 from labelbox.client import Client
 from labelbox.schema.model import Model
 from labelbox.schema.bulk_import_request import BulkImportRequest
-from labelbox.schema.annotation_import import MALPredictionImport, MEAPredictionImport
+from labelbox.schema.annotation_import import MALPredictionImport, MEAPredictionImport, LabelImport
 from labelbox.schema.dataset import Dataset
 from labelbox.schema.data_row import DataRow
 from labelbox.schema.label import Label
@@ -16,7 +16,7 @@ from labelbox.schema.task import Task
 from labelbox.schema.labeling_frontend import LabelingFrontend
 from labelbox.schema.asset_attachment import AssetAttachment
 from labelbox.schema.webhook import Webhook
-from labelbox.schema.ontology import Ontology, OntologyBuilder, Classification, Option, Tool
+from labelbox.schema.ontology import Ontology, OntologyBuilder, Classification, Option, Tool, FeatureSchema
 from labelbox.schema.role import Role, ProjectRole
 from labelbox.schema.invite import Invite, InviteLimit
 from labelbox.schema.data_row_metadata import DataRowMetadataOntology
