@@ -9,6 +9,7 @@ import time
 import ndjson
 from io import StringIO
 import requests
+from collections.abc import Iterable
 
 from labelbox.exceptions import InvalidQueryError, LabelboxError, ResourceNotFoundError, InvalidAttributeError
 from labelbox.orm.db_object import DbObject, Updateable, Deletable
@@ -298,9 +299,10 @@ class Dataset(DbObject, Updateable, Deletable):
                 for key, value in item.items()
             }
 
-        if not isinstance(items, list):
+        if not isinstance(items, Iterable):
             raise ValueError(
-                f"Must pass a list to create_data_rows. Found {type(items)}")
+                f"Must pass an iterable to create_data_rows. Found {type(items)}"
+            )
 
         with ThreadPoolExecutor(file_upload_thread_count) as executor:
             futures = [executor.submit(convert_item, item) for item in items]
@@ -360,7 +362,11 @@ class Dataset(DbObject, Updateable, Deletable):
         return data_rows[0]
 
     def export_data_rows(self, timeout_seconds=120):
-        """ Returns a generator that produces all data rows that are currently attached to this dataset.
+        """ Returns a generator that produces all data rows that are currently
+        attached to this dataset.
+
+        Note: For efficiency, the data are cached for 30 minutes. Newly created data rows will not appear
+        until the end of the cache period.
 
         Args:
             timeout_seconds (float): Max waiting time, in seconds.
